package rdt;

/**
 *  A IntQueue holds a sequence of int objects and provides access to them
 *  in the order that they were added to the sequence.
 */
public class IntQueue {

    private int[] items;
    private int nextAddIndex;
    private int count;

    /**
     *  fixSize() checks if the items array is large enough to hold a new
     *  int.  If not, a new items array is created with twice the capacity
     *  of the old array.  ints are copied from the old array into the new
     *  array starting at index 0.  The variable nextAddIndex is adjusted
     *  accordingly.
     */
    private void fixSize() {
        int oldLength = items.length;
        if (count == oldLength) {
            int[] newItems = new int[2*oldLength];
            // newIndex is used to index the entries in newItems.
            // Since count == oldLength, nextAddIndex should be the
            // index in items of the next entry to come off the queue.
            // Starting there in items, we want to copy the entries into
            // newItems starting at index 0.
            int newIndex = 0;
            for (int i = nextAddIndex; i < oldLength; i++) {
                newItems[newIndex] = items[i];
                newIndex++;
            }
            for (int i = 0; i < nextAddIndex; i++) {
                newItems[newIndex] = items[i];
                newIndex++;
            }
            nextAddIndex = count;
            items = newItems;
        }
    }  // private fixSize()

    /**
     *  new IntQueue() returns a new, empty queue for ints.
     */
    public IntQueue() {
        items = new int[4];
        nextAddIndex = 0;
        count = 0;
    }  // public IntQueue()

    /**
     *  q.getCount() the number of items in q.
     */
    public int getCount() {
        return count;
    }  // public int getCount()

    /**
     *  q.add(i) adds i to q.
     */
    public void add(int i) {
        // Resize the items array if necessary.
        fixSize();
        // Add the item and adjust count and nextAddIndex.
        items[nextAddIndex] = i;
        count++;
        nextAddIndex = (nextAddIndex + 1)%items.length;
    }  // public void add(int)

    /**
     *  q.get() returns the current item of q.
     *  <p>
     *  Precondition: q.getCount() > 0.
     */
    public int get() {
        if (count <= 0) {
            throw new IllegalStateException("Queue is empty.");
        }
        int getIndex = nextAddIndex - count;
        if (getIndex < 0) {
            getIndex += items.length;
        }
        return items[getIndex];
    }  // public int get()

    /**
     *  q.remove() removes the current item of q.
     *  <p>
     *  Precondition: q.getCount() > 0.
     */
    public void remove() {
        if (count <= 0) {
            throw new IllegalStateException("Queue is empty.");
        }
        count--;
    }  // public void remove()

}  // public class IntQueue

