package rdt;

/**
 *  Class RDTTest defines a main method for running the network
 *  simulation using an RDTProtocol as a reliable data transfer
 *  protocol.
 *  <p>
 */
public class RDTTest {

    // MODIFY - change the segmentCount initializer to change the test
    // message length.
    /**
     *  segmentCount is the number of segments to to send across the
     *  connection.
     */
    protected static int segmentCount = 10;

    protected static TCPEntity sender;
    protected static TCPEntity receiver;

    public static void testProtocol() {
        Logger.start();
        for (int i = 0; i < segmentCount; i++) {
            sender.sendData(i);
        }
        sender.disconnect();
    }  // public static void testProtocol()

    /**
     *  The main program tests an RDTProtocol by constructing two
     *  TCPEntities named "S" and "R", setting up a connection between
     *  them, and sending segmentCount segments from the "S" to "R".
     *  Results are recorded by the Logger class.
     */
    public static void main(String[] args) {
        
        if (args.length != 1) {
            System.out.println("Usage: java RDTTest protocolClass");
            System.exit(1);
        }
        Class protocolClass = null;
        RDTProtocol senderProtocol = null;
        RDTProtocol receiverProtocol = null;
        try {
            protocolClass = Class.forName(args[0]);
            senderProtocol = (RDTProtocol)protocolClass.newInstance();
            receiverProtocol = (RDTProtocol)protocolClass.newInstance();
        } catch(ClassNotFoundException ex) {
            System.out.println("RDTTest: protocol class not found.");
            System.exit(1);
        } catch(Exception ex) {
            System.out.println("RDTTest: protocol creation failed.");
            System.exit(1);
        }

        // MODIFY - change the parameters in the next two lines to modify
        // the minimum and maximum round-trip times and the reliability of
        // the internet.
        Internet.setRoundTripTimes(100, 120);
        Internet.setReliability(0.5);
        Internet.printConfiguration();
        System.out.println("Using protocol class " + protocolClass.getName());

        System.out.println("Message sent by S:");
        System.out.println("------------------");
        for (int i = 0; i < segmentCount; i++) {
            System.out.println("Segment " + i);
        }
        System.out.println();

        sender = new TCPEntity("S");
        receiver = new TCPEntity("R");
        Thread senderThread = sender.setUp(receiver, senderProtocol);
        Thread receiverThread = receiver.setUp(sender, receiverProtocol);
        testProtocol();

        try {
            receiverThread.join();
            senderThread.join();
        } catch(InterruptedException e) {
            e.printStackTrace(System.err);
            System.exit(1);
        }
        System.out.println();
        System.out.println("Message Received by R:");
        System.out.println("----------------------");
        System.out.print(receiver.getReceivedMessage());
        System.out.println();
        System.exit(0);

    }  // public static void main(String[])

}  // public class RDTTest

