package rdt;

import java.awt.event.*;
import javax.swing.*;

/**
 *  <p>
 *  An RDTTimer schedules a timeout task for a packet to be executed in
 *  the future.
 *  It allows the scheduling time for the task to be changed to a later
 *  time and it also allows the scheduling time to cancelled.
 *  Thus an RDTTimer can be reused as many times as necessary.
 *  </p>
 *  <p>
 *  When constructed, an RDTTimer is provided with a delay and a task.
 *  These are normally set by a TCPEntity.
 *  Before it is used is should be given a packet by invoking setPacket().
 *  This packet is made available when the timer fires.
 *  A <code>null</code> value can be used for the packet if only one
 *  timer is used by a protocol.
 *  </p>
 *  <p>
 *  When the current time is equal to its scheduling time, an RDTTimer
 *  schedules its task.
 *  Its packet is used as the parameter for its task's
 *  <code>doTask()</code> method.
 *  </p>
 *  <p>
 *  When constructed, a RDTTimer has no scheduling time.
 *  It can be given a scheduling time by sending it a start() message,
 *  which sets the scheduling time to the current time plus its delay.
 *  The scheduling time can be cancelled by sending it a stop() message.
 *  </p>
 */
public abstract class RDTTimer {

    private Packet packet;

    /**
     *  tmr.getPacket() returns the packet that tmr times acknowlegements
     *  for.
     */
    public Packet getPacket() {
        return packet;
    }  // public Packet getPacket()

    /**
     *  tmr.setPacket(pkt) sets the packet that tmr times acknowlegements
     *  for to pkt.
     */
    public void setPacket(Packet pkt) {
        packet = pkt;
    }  // public void setPacket(Packet)

    /**
     *  tmr.start() sets the scheduling time for tmr to the current time
     *  plus tmr's delay.
     *  At the scheduling time, a task for tmr's packet will be scheduled
     *  on tmr's queue, unless tmr.start() or tmr.stop() has been invoked
     *  in the mean time.
     */
    public abstract void start();

    /**
     *  tmr.start(del) sets the scheduling time for tmr to the current
     *  time plus del.
     *  At the scheduling time, a task for tmr's packet will be scheduled
     *  on tmr's queue, unless tmr.start() or tmr.stop() has been invoked
     *  in the mean time.
     */
    public abstract void start(long del);

    /**
     *  tmr.stop() cancels tmr's scheduling time.
     */
    public abstract void stop();

}  // public class RDTTimer

