package rdt;

public class TimerFactoryTest {

    private static final int DELAY = 50;
    private static final int TOLERANCE = 5;

    TimerFactory testTimerFactory;
    TimerFactory.Timer testTimer;

    TimerFactory controlTimerFactory;
    TimerFactory.Timer startTimer;
    TimerFactory.Timer stopTimer;

    int[] delays;
    int index;

    long fireTime;
    boolean stopped;
    boolean error;

    private synchronized void checkMissedFire() {
        long currentTime = System.currentTimeMillis();
        if (!stopped && fireTime + TOLERANCE < currentTime) {
            Logger.log("MISSED FIRE at time " + fireTime);
            error = true;
        }
    }  // private synchronized void checkMissedFire()

    private synchronized void checkFalseFire() {
        long currentTime = System.currentTimeMillis();
        if (stopped || fireTime > currentTime + TOLERANCE) {
            Logger.log("FALSE FIRE");
            error = true;
        }
    }  // private synchronized void checkFalseFire()

    private synchronized void setStopped() {
        stopped = true;
    }  // private synchronized void setStopped()

    private synchronized void setFireTime(int del) {
        fireTime = System.currentTimeMillis() + del;
        stopped = false;
    }  // private synchronized void setFireTime(int)

    public TimerFactoryTest() {
        testTimerFactory = new TimerFactory("Test");
        testTimer = testTimerFactory.createTimer(DELAY, new TimingTask() {
            public void doTask(Packet pkt) {
                checkMissedFire();
                checkFalseFire();
                setStopped();
                Logger.log("Test timer fired");
            }
        });
        controlTimerFactory = new TimerFactory("Control");
        startTimer = controlTimerFactory.createTimer(DELAY, new TimingTask() {
            public void doTask(Packet pkt) {
                checkMissedFire();
                Logger.log("Start timer fired");
                if (index < delays.length) {
                    int delay = delays[index];
                    setFireTime(delay);
                    testTimer.start(delay);
                    Logger.log("Test timer started with delay " + delay);
                    index++;
                }
                if (index < delays.length) {
                    startTimer.start();
                    Logger.log("Start timer started");
                }
            }
        });
        stopTimer = controlTimerFactory.createTimer(DELAY, new TimingTask() {
            public void doTask(Packet pkt) {
                checkMissedFire();
                Logger.log("Stop timer fired");
                setStopped();
                testTimer.stop();
                Logger.log("Test timer stopped");
            }
        });
    }

    public void doTest(int[] dels) {
        delays = dels;
        if (dels == null) {
            System.out.println("TEST SETUP ERROR: null delay array");
            System.out.println();
            return;
        }
        if (dels.length == 0) {
            System.out.println("TEST SETUP ERROR: empty delay array");
            System.out.println();
            return;
        }
        index = 0;
        stopped = true;
        error = false;
        Logger.start();
        Logger.log("Timer default delay is " + DELAY);
        int stopDelay = delays[index];
        index++;
        Logger.log("Stop delay is " + stopDelay);
        stopTimer.start(stopDelay);
        Logger.log("Stop timer started");
        startTimer.start(0);
        Logger.log("Start timer started with delay 1");
        controlTimerFactory.waitUntilStopped();
        Logger.log("Control timers stopped");
        testTimerFactory.waitUntilStopped();
        String status = error ? "FAILED" : "PASSED";
        Logger.log("All timers stopped, TEST " + status);
        Logger.println();
    }  // public void doTest()

    public static void main(String[] args) {
        TimerFactoryTest tester = new TimerFactoryTest();
        tester.doTest(new int[] {240, 55, 55, 55, 55, 55});
        tester.doTest(new int[] {240, 55, 55, 55, 55, 55, 55});
        tester.doTest(new int[] {500, 200, 10, 200, 10, 200, 10});
        tester.doTest(new int[] {240, 50, 50, 50, 50, 50});
    }  // public static void main(String[])

}  // public class TimerFactoryTest

